/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.client.client.ui.sinfo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.SortedMap;
import java.util.TreeMap;

import org.dive4elements.river.client.client.FLYSConstants;
import org.dive4elements.river.client.client.ui.AbstractUIProvider;
import org.dive4elements.river.client.client.ui.ParameterMatrix.Column;
import org.dive4elements.river.client.shared.model.Data;
import org.dive4elements.river.client.shared.model.DataItem;
import org.dive4elements.river.client.shared.model.DataList;
import org.dive4elements.river.client.shared.model.DefaultData;
import org.dive4elements.river.client.shared.model.DefaultDataItem;
import org.dive4elements.river.client.shared.model.MultiAttributeData;
import org.dive4elements.river.client.shared.model.MultiDataItem;

import com.google.gwt.core.client.GWT;
import com.google.gwt.i18n.client.NumberFormat;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.ListGridFieldType;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.TextItem;
import com.smartgwt.client.widgets.form.validator.IsIntegerValidator;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;

public class CollisionLoadYearPanel extends AbstractUIProvider {

    public interface IColumnClickHandler {
        void columnClicked(String value);
    }

    private enum PinType {
        none, from, to
    }

    private final FLYSConstants MESSAGES = GWT.create(FLYSConstants.class);

    private TextItem yearsItem;

    private final List<String> validYears = new ArrayList<String>();

    private ListGrid table;

    public CollisionLoadYearPanel() {
    }

    @Override
    public Canvas createOld(final DataList dataList) {
        final List<Data> items = dataList.getAll();
        final Data years = getData(items, "years");
        final DataItem[] yearsItems = years.getItems();

        final String v1 = yearsItems[0].getStringValue().replace(" ", ", ");

        final Label old = new Label(v1);
        final HLayout layout = new HLayout();
        layout.setWidth("400px");

        final Label label = new Label(dataList.getLabel());
        label.setWidth("200px");

        final Canvas back = getBackButton(dataList.getState());

        layout.addMember(label);
        layout.addMember(old);
        layout.addMember(back);

        return layout;
    }

    @Override
    public List<String> validate() {
        return validateYears();
    }

    @Override
    public Canvas create(final DataList data) {
        final VLayout layout = new VLayout();

        final Canvas helper = createHelper(data);

        this.helperContainer.addMember(helper);

        final Canvas submit = getNextButton();
        final Canvas widget = createWidget(data);

        layout.addMember(widget);
        layout.addMember(submit);

        return layout;
    }

    private Canvas createHelper(final DataList dataList) {
        this.table = new ListGrid();
        this.table.setShowHeaderContextMenu(false);
        this.table.setWidth100();
        this.table.setShowRecordComponents(true);
        this.table.setShowRecordComponentsByCell(true);
        this.table.setHeight100();
        this.table.setEmptyMessage(this.MSG.empty_table());
        this.table.setCanReorderFields(false);

        final MultiAttributeData mData = findDefinition(dataList, "years");
        final Map<String, Map<String, String>> meta = mData.getMeta();

        final Map<String, String> tableInfo = meta.get("meta_tableinfo");
        final String keyColumnName = tableInfo.get("keycol");

        final IColumnClickHandler fromHandler = new IColumnClickHandler() { // add to external class

            @Override
            public void columnClicked(final String value) {
                appendYear(value);
            }
        };

        final SortedMap<Integer, ListGridField> fields = new TreeMap<Integer, ListGridField>();

        for (final Entry<String, Map<String, String>> entry : meta.entrySet()) {

            final String colName = entry.getKey();
            if (colName.startsWith("meta_"))
                continue;

            final Map<String, String> values = entry.getValue();

            final int order = Integer.parseInt(values.get("order"));

            final ListGridField field = createField(colName, keyColumnName, values, fromHandler);

            fields.put(order, field);
        }

        for (final DataItem dataItem : mData.opts) {
            final MultiDataItem item = (MultiDataItem) dataItem;

            final Map<String, String> valueMap = item.getValue();
            // final String columnName = mData.getLabel();
            final Record newRecord = new Record();
            for (final Entry<String, String> entry : valueMap.entrySet()) {
                final String key = entry.getKey();
                final String value = entry.getValue();
                newRecord.setAttribute(key, value);

                if (key.equals(keyColumnName))
                    this.validYears.add(value);
            }

            this.table.addData(newRecord);
        }

        this.table.setFields(fields.values().toArray(new ListGridField[fields.size()]));

        return this.table;
    }

    private MultiAttributeData findDefinition(final DataList dataList, final String paranemterName) {

        for (final Data data : dataList.getAll()) {

            final String label = data.getLabel();
            if (label.equals(paranemterName) && data instanceof MultiAttributeData)
                return (MultiAttributeData) data;
        }

        return null;
    }

    private ListGridField createField(final String colName, final String keyColumnName, final Map<String, String> values,
            final IColumnClickHandler fromHandler) {

        final String translation = getMeta(values, "translation", colName);

        final ListGridField field = new ListGridField(colName, translation);

        final String width = getMeta(values, "width", "99");
        field.setWidth(width);

        final ListGridFieldType type = getMeta(values, "type", ListGridFieldType.TEXT, ListGridFieldType.class);
        field.setType(type);

        final Alignment alignment = getMeta(values, "alignment", Alignment.LEFT, Alignment.class);
        field.setAlign(alignment);

        final PinType pinType = getMeta(values, "pin", PinType.none, PinType.class);
        switch (pinType) {
        case from: {
            final String baseUrl = GWT.getHostPageBaseURL();
            field.setCellIcon(baseUrl + this.MESSAGES.markerGreen());
            field.addRecordClickHandler(new RecordClickHandler() {
                @Override
                public void onRecordClick(final RecordClickEvent e) {
                    final Record r = e.getRecord();
                    fromHandler.columnClicked(r.getAttribute(keyColumnName));
                }
            });
        }
            break;

        case to:
            break;

        case none:
        default:
            break;
        }

        return field;
    }

    private String getMeta(final Map<String, String> values, final String key, final String defaultValue) {

        final String value = values.get(key);
        if (value == null || value.isEmpty())
            return defaultValue;

        return value;
    }

    private <TYPE extends Enum<TYPE>> TYPE getMeta(final Map<String, String> values, final String key, final TYPE defaultValue, final Class<TYPE> enumType) {

        final String value = values.get(key);
        if (value == null || value.isEmpty())
            return defaultValue;

        try {
            return Enum.valueOf(enumType, value);
        }
        catch (final Exception e) {
            e.printStackTrace();
            return defaultValue;
        }
    }

    private Map<String, Column> columns;
    private List<String> columnNames;
    private List<String> valueNames;
    private Map<String, List<String>> attributes;
    private Map<String, List<String>> selected;

    public void addColumn(final MultiAttributeData options) {
        GWT.log("Add Columns for MultiAttribute data");

        this.columns = new HashMap<String, Column>();
        this.columnNames = new ArrayList<String>();
        this.valueNames = new ArrayList<String>();
        this.selected = new HashMap<String, List<String>>();
        this.attributes = new HashMap<String, List<String>>();

        final String groupTitle = options.getLabel();

        final Column col = new Column(groupTitle);
        final DataItem[] items = options.getItems();

        if (items == null) {
            GWT.log("No items found in StringOptionsData '" + groupTitle + "'");
            return;
        }

        final MultiDataItem mItem = (MultiDataItem) items[0];
        for (final Map.Entry<String, String> entry : mItem.getValue().entrySet()) {
            if (entry.getKey().equals("art:value") || entry.getKey().equals("art:label")) {
                continue;
            }
            this.attributes.put(entry.getKey(), new ArrayList<String>());
        }
        for (final DataItem item : items) {
            GWT.log("multidataitem: " + item.getLabel());
            final String title = item.getLabel();

            if (this.valueNames.indexOf(title) < 0) {
                this.valueNames.add(title);
            }
            final MultiDataItem mi = (MultiDataItem) item;
            final Map<String, String> vs = mi.getValue();
            for (final Map.Entry<String, String> e : vs.entrySet()) {
                if (e.getKey().equals("art:value") || e.getKey().equals("art:label")) {
                    continue;
                }
                final List<String> data = this.attributes.get(e.getKey());
                data.add(e.getValue());
            }
            col.addValue(item.getLabel(), mi.getValue().get("art:value"));
        }

        this.columnNames.add(groupTitle);
        this.columns.put(groupTitle, col);
    }

    public Canvas createWidget(final DataList data) {
        final VLayout layout = new VLayout();

        final Label title = new Label(data.get(0).getDescription());
        title.setHeight("25px");

        final DynamicForm form = new DynamicForm();
        form.setNumCols(4);
        this.yearsItem = new TextItem(this.MSG.years());
        this.yearsItem.setValidators(new IsIntegerValidator());
        form.setFields(this.yearsItem);

        layout.addMember(title);
        layout.addMember(form);

        return layout;
    }

    @Override
    protected Data[] getData() {
        validateYears();
        if (this.yearsItem != null && !this.yearsItem.getValueAsString().isEmpty()) {
            final List<Data> data = new ArrayList<Data>();

            final DataItem yearsdata = new DefaultDataItem("years", "years", this.yearsItem.getValueAsString().trim());
            data.add(new DefaultData("years", null, null, new DataItem[] { yearsdata }));

            return data.toArray(new Data[data.size()]);
        }
        return new Data[0];
    }

    private List<String> validateYears() {
        final List<String> errors = new ArrayList<String>();
        final NumberFormat nf = NumberFormat.getDecimalFormat();

        if (this.yearsItem.getValueAsString() == null || this.yearsItem.getValueAsString().trim().isEmpty()) {
            errors.add(this.MESSAGES.empty_filter());
            return errors;
        }

        final String[] sValues = this.yearsItem.getValueAsString().trim().split(" ");
        String filtered = "";
        int goodValues = 0;
        for (final String sValue : sValues) {
            int value;
            try {
                value = Integer.parseInt(sValue);
            }
            catch (final NumberFormatException e) {
                errors.add(this.MESSAGES.wrongFormat() + ": " + sValue);
                continue;
            }
            boolean isGood = false;
            for (final String validYear : this.validYears) {
                /* No list contains for strings? */
                if (sValue.equals(validYear)) {
                    isGood = true;
                    break;
                }
            }
            if (!isGood) {
                String tmp = this.MESSAGES.no_data_for_year();
                tmp = tmp.replace("$1", sValue);
                errors.add(tmp);
                continue;
            }
            goodValues++;
            if (goodValues > 1) {
                filtered += " " + Integer.toString(value);
            } else {
                filtered = Integer.toString(value);
            }
        }

        return errors;
    }

    protected final void appendYear(final String year) {
        final String oldYears = this.yearsItem.getValueAsString();
        if (oldYears != null && !oldYears.isEmpty()) {
            this.yearsItem.setValue(oldYears.trim() + " " + year);
        } else {
            this.yearsItem.setValue(year);
        }
    }
}
